# ------------------------------------------------------------------------------
#  Copyleft 2015-2021  PacMiam
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
# ------------------------------------------------------------------------------

# Filesystem
from pathlib import Path
from shutil import rmtree

# Geode
from geode_gem.engine.api import GEM
from geode_gem.engine.console import Console
from geode_gem.engine.emulator import Emulator
from geode_gem.engine.game import Game
from geode_gem.engine.lib.database import Database

# System
from tempfile import mkdtemp

# Unittest
import unittest
from unittest import mock


# ------------------------------------------------------------------------------
#   Tests
# ------------------------------------------------------------------------------

class GeodeGEMTC(unittest.TestCase):

    def setUp(self):
        """ Initialize each test with some data
        """

        self.temporary_directory = Path(mkdtemp(prefix="geode-", suffix="-gem"))

        self.config_directory = self.temporary_directory.joinpath("config")
        self.local_directory = self.temporary_directory.joinpath("local")

        self.api = GEM(self.config_directory, self.local_directory)

    def tearDown(self):
        """ Remove data from initialization when test terminate
        """

        rmtree(self.temporary_directory)

    def test_api_constructor(self):
        """ Check geode_gem.engine.api.GEM contructor
        """

        self.assertTrue(self.api.get_config().exists())
        self.assertTrue(self.api.get_local().exists())
        self.assertFalse(self.api.is_locked())
        self.assertFalse(self.api.debug)

        for filename in ("gem.log", "gem.db", ".lock"):
            path = self.api.get_local(filename)

            self.assertTrue(path.exists())
            self.assertEqual(
                path, self.local_directory.joinpath(filename))

        self.assertEqual(self.api.database.path, self.api.get_local("gem.db"))

    def test_api_lock_manager(self):
        """ Check geode_gem.engine.api.GEM.is_locked method
        """

        self.assertFalse(self.api.is_locked())

        second_instance = GEM(self.config_directory, self.local_directory)
        self.assertTrue(second_instance.is_locked())

        # Check if database is correctly ignored
        with self.assertRaises(AttributeError):
            second_instance.database

    @mock.patch.object(Database, "check_integrity")
    def test_api_initialization(self, mock_object):
        """ Check geode_gem.engine.api.GEM.init method
        """

        self.api.init()
        self.assertEqual(len(self.api.consoles), 0)
        self.assertEqual(len(self.api.emulators), 0)

        # Simulate migration
        self.api.database.modify(
            "gem", {"version": "0.1.0"}, {"version": self.api.Version})
        self.api.free_lock()
        # Simulate database integrity problem
        mock_object.return_value = False

        api = GEM(self.config_directory, self.local_directory)
        with self.assertRaises(RuntimeError):
            api.init()

    def test_api_write_data(self):
        """ Check geode_gem.engine.api.GEM.write_data method
        """

        filenames = (GEM.Consoles, GEM.Emulators)
        for filename in filenames:
            self.assertFalse(self.api.get_config(filename).exists())

        self.api.add_console("test", ())
        self.api.write_data(*filenames)

        self.api.add_emulator("test", ())

        self.assertEqual(len(self.api.consoles), 1)
        self.assertEqual(len(self.api.emulators), 1)

        # The write_data method write the specified files even is there is no
        # content inside
        for filename in filenames:
            self.assertTrue(self.api.get_config(filename).exists())

        # Emulator has been added after the write_data, so the emulators.conf
        # must be empty
        with self.api.get_config(GEM.Emulators).open('r') as pipe:
            self.assertEqual(pipe.read(), str())

    def test_api_add_console(self):
        """ Check geode_gem.engine.api.GEM.add_console method
        """

        console = self.api.add_console("test", ())

        self.assertIs(type(console), Console)
        self.assertEqual(len(self.api.consoles), 1)

        self.assertEqual(self.api.get_console("test").id, console.id)
        self.assertIsNone(self.api.get_console("unkwnown"))

    def test_api_update_console(self):
        """ Check geode_gem.engine.api.GEM.update_console method
        """

        console = self.api.add_console("test", ())
        console.icon = "beautiful.png"

        self.assertNotEqual(
            self.api.get_console(console.id).icon, console.icon)

        self.api.update_console(console)
        self.assertEqual(
            self.api.get_console(console.id).icon, console.icon)

    def test_api_replace_console(self):
        """ Check geode_gem.engine.api.GEM.replace_console method
        """

        with self.assertRaises(TypeError):
            self.api.replace_console("unknown-test", "whatever")

        console = self.api.add_console("test", ())
        console2 = self.api.add_console("test-2", ())

        self.assertIsNotNone(self.api.get_console("test"))
        self.api.replace_console(console.id, console2)

        self.assertIsNone(self.api.get_console("test"))
        self.assertIsNotNone(self.api.get_console("test-2"))

    def test_api_delete_console(self):
        """ Check geode_gem.engine.api.GEM.delete_console method
        """

        console = self.api.add_console("test", ())

        self.api.delete_console(console.id)
        with self.assertRaises(IndexError):
            self.api.delete_console(console.id)

    def test_api_add_emulator(self):
        """ Check geode_gem.engine.api.GEM.add_emulator method
        """

        emulator = self.api.add_emulator("test", ())

        self.assertIs(type(emulator), Emulator)
        self.assertEqual(len(self.api.emulators), 1)

        self.assertEqual(self.api.get_emulator("test").id, emulator.id)
        self.assertIsNone(self.api.get_emulator("unkwnown"))

    def test_api_update_emulator(self):
        """ Check geode_gem.engine.api.GEM.update_emulator method
        """

        emulator = self.api.add_emulator("test", ())
        emulator.binary = "python"

        self.assertNotEqual(
            self.api.get_emulator(emulator.id).binary, emulator.binary)

        self.api.update_emulator(emulator)
        self.assertEqual(
            self.api.get_emulator(emulator.id).binary, emulator.binary)

    def test_api_replace_emulator(self):
        """ Check geode_gem.engine.api.GEM.replace_emulator method
        """

        with self.assertRaises(TypeError):
            self.api.replace_emulator("unknown-test", "whatever")

        emulator = self.api.add_emulator("test", ())
        emulator2 = self.api.add_emulator("test-2", ())

        self.assertIsNotNone(self.api.get_emulator("test"))
        self.api.replace_emulator(emulator.id, emulator2)

        self.assertIsNone(self.api.get_emulator("test"))
        self.assertIsNotNone(self.api.get_emulator("test-2"))

    def test_api_delete_emulator(self):
        """ Check geode_gem.engine.api.GEM.delete_emulator method
        """

        emulator = self.api.add_emulator("test", ())

        self.api.delete_emulator(emulator.id)
        with self.assertRaises(IndexError):
            self.api.delete_emulator(emulator.id)

    def test_api_update_game(self):
        """ Check geode_gem.engine.api.GEM.update_game method
        """

        self.assertIsNone(self.api.database.select("games", '*'))

        path = self.temporary_directory.joinpath("power.cell")
        path.touch()

        game = Game(self.api, path)
        self.api.update_game(game)

        database_games = self.api.database.select("games", 'name')
        self.assertEqual(len(database_games), 1)
        self.assertEqual(database_games[0], "power")

        game.name = "Another name"
        self.api.update_game(game)

        database_games = self.api.database.select("games", 'name')
        self.assertEqual(len(database_games), 1)
        self.assertEqual(database_games[0], "Another name")

        path2 = self.temporary_directory.joinpath("romaric.tix")
        path2.touch()

        game2 = Game(self.api, path2)
        self.api.update_game(game2)

        database_games = self.api.database.select("games", 'name')
        self.assertEqual(len(database_games), 2)
        self.assertEqual(database_games[1], "romaric")

    def test_api_get_game_tags(self):
        """ Check geode_gem.engine.api.GEM.get_game_tags method
        """

        self.assertListEqual(self.api.get_game_tags(), list())

        path = self.temporary_directory.joinpath("awesome_game.ext")
        path.touch()

        game = Game(self.api, path)
        game.tags = ["so fun", "amazing"]

        self.api.update_game(game)
        self.assertListEqual(self.api.get_game_tags(), sorted(game.tags))

        path2 = self.temporary_directory.joinpath("incredible_game.ext")
        path2.touch()

        game2 = Game(self.api, path2)
        game2.tags = ["incredible", "so fun"]

        self.api.update_game(game2)
        self.assertListEqual(
            self.api.get_game_tags(), ["amazing", "incredible", "so fun"])
