# ------------------------------------------------------------------------------
#  Copyleft 2015-2021  PacMiam
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
# ------------------------------------------------------------------------------

# Datetime
from datetime import timedelta

# Filesystem
from pathlib import Path

# Geode
from geode_gem.engine.console import Console
from geode_gem.engine.emulator import Emulator
from geode_gem.engine.game import Game
from geode_gem.engine.utils import generate_identifier

# System
from tempfile import gettempdir

# Unittest
import unittest


# ------------------------------------------------------------------------------
#   Tests
# ------------------------------------------------------------------------------

class GeodeGEMGameTC(unittest.TestCase):

    def setUp(self):
        """ Initialize each test with some data
        """

        self.path = Path(gettempdir(), "awesome_game.ext")
        self.path.touch()

        self.game = Game(None, self.path)

    def tearDown(self):
        """ Remove data from initialization when test terminate
        """

        if self.path.exists():
            self.path.unlink()

    def test_game_copy(self):
        """ Check geode_gem.engine.game.Game.copy method
        """

        with self.assertRaises(FileNotFoundError):
            self.game.copy(Path(gettempdir(), "unexisting_game.txt"))

        path = Path(gettempdir(), "duplicate_game.txt")
        path.touch()

        game = self.game.copy(path)

        self.assertEqual(game.id, generate_identifier(path))
        self.assertEqual(game.name, path.stem)

        path.unlink()

    def test_game_reset(self):
        """ Check geode_gem.engine.game.Game.reset method
        """

        self.game.name = "This is an awesome game, yay !"
        self.game.score = 5
        self.game.multiplayer = True

        self.game.reset()

        self.assertEqual(self.game.name, self.path.stem)
        self.assertEqual(self.game.score, 0)
        self.assertFalse(self.game.multiplayer)

    def test_game_extension(self):
        """ Check geode_gem.engine.game.Game.extension property
        """

        data = [("awesome_game.ext", ".ext"),
                ("multi_extensions.tar.xz", ".xz"),
                ("damn.this_is_a.strange_name.what", ".what")]

        for filename, extension in data:
            path = Path(gettempdir(), filename)
            path.touch()

            self.assertEqual(Game(None, path).extension, extension)
            path.unlink()

    def test_game_as_dict(self):
        """ Check geode_gem.engine.game.Game.as_dict method
        """

        self.game.play_time = timedelta(seconds=42)
        self.game.tags = ["platform", "action"]

        data = self.game.as_dict()

        self.assertEqual(sorted(data.keys()), [
            "arguments", "cover", "emulator", "favorite", "filename", "finish",
            "key", "last_play", "last_play_time", "multiplayer", "name",
            "play", "play_time", "score", "tags"])

        self.assertEqual(data["play_time"], "00:00:42")
        self.assertEqual(data["name"], self.game.name)
        self.assertEqual(data["tags"], "platform;action")

    def test_game_command(self):
        """ Check geode_gem.engine.game.Game.command method
        """

        emulator = Emulator(name="Test", binary="python")

        self.assertIsNone(self.game.command())

        self.game.emulator = emulator
        self.assertEqual(self.game.command(), ["python", str(self.path)])

        emulator.windowed = "--windowed"
        self.assertEqual(self.game.command(),
                         ["python", "--windowed", str(self.path)])

        self.assertEqual(self.game.command(fullscreen=True),
                         ["python", str(self.path)])

        emulator.fullscreen = "--fullscreen"
        self.assertEqual(self.game.command(fullscreen=True),
                         ["python", "--fullscreen", str(self.path)])

    def test_game_has_custom_parameters(self):
        """ Check geode_gem.engine.game.Game.has_custom_parameters method
        """

        emulator = Emulator(name="Test")

        self.assertFalse(self.game.has_custom_parameters(None))

        console = Console(None, name="Test", emulator=emulator)
        self.assertFalse(self.game.has_custom_parameters(console))

        self.game.emulator = emulator
        self.assertFalse(self.game.has_custom_parameters(console))

        self.game.emulator = Emulator(name="Test2")
        self.assertTrue(self.game.has_custom_parameters(console))

        self.game.emulator = None
        self.game.default = "blah"
        self.assertTrue(self.game.has_custom_parameters(console))

        self.game.default = str()
        self.game.environment["DISPLAY"] = ":0"
        self.assertTrue(self.game.has_custom_parameters(console))
